#include <SPI.h>

#include <SD.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 20, 4);

int state = 0;
const int8_t buttonPin[] = {3, 4, 5, 6, 2};
int buttonState[] = {0, 0, 0, 0, 0};
int8_t tempo[] = {75, 111};
char *files[] = {"game", "pink"};
char *names[] = {"Game of Thrones", "Pink Panther"};

int nrSongs = 2, newSongs = 0;
bool shuffle = 0;
int buzzer1 = 8;
int wholenote, divider = 0, noteDuration = 0;

File myFile;

void play() {
  for (int i = 0; i < nrSongs ; i++) {
    int j = i;
    if(shuffle) {
      j = random(nrSongs);
    }
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print(names[j]);
    lcd.setCursor(0,2);
    lcd.print("<-----Next");
    lcd.setCursor(0,3);
    lcd.print("      Back--->");
    char file[40];
    strcpy(file, files[j]);
    strcat(file, ".txt");
    myFile = SD.open(file);
    wholenote = (60000 * 4) / tempo[j];
    char nota[5], timp[3], c;
    while(myFile.available()) {
       buttonState[0] = digitalRead(buttonPin[0]);
       if(buttonState[0] == HIGH) {
          myFile.close();
          return;
       }
       buttonState[1] = digitalRead(buttonPin[1]);
       if(buttonState[1] == HIGH) {
          myFile.close();
          break;
       }
       j = 0;
       c = myFile.read();
       while(c != 10) {
         nota[j] = c;
         j++;
         c = myFile.read();
       }
       nota[j] = '\0';
       j = 0;
       c = myFile.read();
       if(c == '-') {
         c = myFile.read();
         while(c != 10) {
           timp[j] = c;
           j++;
           c = myFile.read();
         }
         timp[j] = '\0';
         noteDuration = (wholenote) / atoi(timp);
         noteDuration *= 1.5;
       } else {
         timp[j] = c;
         j++;
         c = myFile.read();
         while(c != 10) {
           timp[j] = c;
           j++;
           c = myFile.read();
         }
         timp[j] = '\0';
         noteDuration = (wholenote) / atoi(timp);
       }
       tone(buzzer1, atoi(nota), noteDuration * 0.9);
       delay(noteDuration);
       noTone(buzzer1);
     }
     myFile.close();
   }
}

void record() {
  int melody[50], i, j = 0;
  while(!buttonState[1]) {
    buttonState[0] = digitalRead(buttonPin[0]);
    buttonState[1] = digitalRead(buttonPin[1]);
    if(buttonState[0] == HIGH) {
      break;
    }
    for(i = 2; i < 5; i++) {
      buttonState[i] = digitalRead(buttonPin[i]);
      if(buttonState[i] == HIGH) {
        melody[j] = i * 100;
        j++;
        melody[j] = 4;
        j++;
      }
    }
  }
  char file[10];
  sprintf(file, "NewSong%d", newSongs);
  newSongs++;
  files[nrSongs] = file;
  names[nrSongs] = file;
  tempo[nrSongs] = 90;
  nrSongs++; 
  myFile = SD.open(file, FILE_WRITE);
  for(i = 0; i < j; i++) {
    myFile.println(melody[i]);
  }
  myFile.close();
}

void setup() {
  
  //initialize lcd screen
  lcd.begin();
  // turn on the backlight
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Hello to JukeBox");
  lcd.setCursor(0,1);
  lcd.print("Press for start");

  for (int i = 0; i < 5; i++) {
    pinMode(buttonPin[i], INPUT);
  }
  pinMode(13, OUTPUT);

  Serial.begin(9600);
  while (!Serial) {
  }
  Serial.print("Initializing SD card...");
  if (!SD.begin(10)) {
    Serial.println("initialization failed!");
    while (1);
  }
  Serial.println("initialization done.");
}

void loop() {
  delay(200);
  buttonState[0] = digitalRead(buttonPin[0]);
  if(buttonState[0] == HIGH) {
    switch(state) {
      case 0:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("<----Play");
        lcd.setCursor(0,1);
        lcd.print("         Record---->");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        state = 1;
        break;
      case 1:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("Hello to JukeBox");
        lcd.setCursor(0,1);
        lcd.print("Press for menu");
        state = 0;
        break;
      case 2:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("<----Play");
        lcd.setCursor(0,1);
        lcd.print("         Record---->");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        state = 1;
        break;
      case 3:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("<----Play");
        lcd.setCursor(0,1);
        lcd.print("         Record---->");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        state = 1;
        break;
      case 4:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("<----Shuffle Play");
        lcd.setCursor(0,1);
        lcd.print("<----Play");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        state = 3;
        break;
      default:
        break;
    }
  }
  buttonState[1] = digitalRead(buttonPin[1]);
  if(buttonState[1] == HIGH) {
    switch(state) {
      case 1:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("-----Recording-----");
        lcd.setCursor(0,1);
        lcd.print("<----Save");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        record();
        state = 2;
        break;
      default:
        break;
    }
  }
  buttonState[2] = digitalRead(buttonPin[2]);
  if(buttonState[2] == HIGH) {
    switch(state) {
      case 1:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("<-----Shuffle");
        lcd.setCursor(0,1);
        lcd.print("<-----Play");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        state = 3;
        break;
      case 2:
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("<----Play");
        lcd.setCursor(0,1);
        lcd.print("         Record---->");
        lcd.setCursor(0,2);
        lcd.print("           Back---->");
        state = 1;
        break;
      case 3:
        shuffle = 1;
        play();
        shuffle = 0;
        state = 4;
        break;
      default:
        break;
    }
  }
  buttonState[3] = digitalRead(buttonPin[3]);
  if(buttonState[3] == HIGH) {
    switch(state) {
      case 3:
        play();
        state = 4;
        break;
      default:
        break;
    }
  }
}
